/*-----------------------------------------------------------------------

File  : cto_proofstate.c

Author: Stephan Schulz

Contents
 
  Basic functions for proof objects.

Changes

<1> Mon Jun  8 11:14:02 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_proofstate.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: ProofStateAlloc()
//
//   Return an empty, initialized proof state.
//
// Global Variables: -
//
// Side Effects    : memory operations
//
/----------------------------------------------------------------------*/

ProofState_p ProofStateAlloc()
{
   ProofState_p handle = ProofStateCellAlloc();
   PStack_p pdummy = PStackAlloc();
   DStack_p ddummy = DStackAlloc();

   handle->signature           = SigAlloc();
   handle->ordering            = NULL;
   handle->heuristic           = NULL;
   handle->original_terms      = TBAlloc(TPIgnoreProps, handle->signature);
   handle->terms               = TBAlloc(TPRestricted, handle->signature);
   handle->tmp_terms           = TBAlloc(TPIgnoreProps, handle->signature);
   handle->freshvars           = VarBankAlloc();
   handle->axioms              = ClauseSetAlloc();
   handle->processed_pos_rules = ClauseSetAlloc();
   handle->processed_pos_eqns  = ClauseSetAlloc();
   handle->processed_neg_units = ClauseSetAlloc();
   handle->processed_non_units = ClauseSetAlloc();
   handle->unprocessed         = ClauseSetAlloc();
   handle->tmp_store           = ClauseSetAlloc();
   handle->demods[0]           = handle->processed_pos_rules;
   handle->demods[1]           = handle->processed_pos_eqns;
   handle->demods[2]           = NULL;
   handle->paramod_with_units_only = false;
   handle->prefer_initial_clauses = false;
   handle->processed_count     = 0;
   handle->trivial_count       = 0; 
   handle->forward_subsumed_count = 0; 
   handle->non_trivial_count = 0;
   handle->backward_subsumed_count =0;
   handle->backward_rewritten_count =0;
   handle-> generated_count  = 0;
   handle->non_trivial_generated_count = 0;
   handle->paramod_count     = 0;
   handle->factor_count      = 0;
   handle->resolv_count      = 0;

   PStackFree(pdummy);
   DStackFree(ddummy);

   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: ProofStateFree()
//
//   Free a ProofStateCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void ProofStateFree(ProofState_p junk)
{
   assert(junk);
   if(junk->ordering)
   {
      OCBFree(junk->ordering);
   }
   if(junk->heuristic)
   {
      HCBFree(junk->heuristic);
   }
   ClauseSetFree(junk->axioms);
   ClauseSetFree(junk->processed_pos_rules);
   ClauseSetFree(junk->processed_pos_eqns);
   ClauseSetFree(junk->processed_neg_units);
   ClauseSetFree(junk->processed_non_units);
   ClauseSetFree(junk->unprocessed);
   ClauseSetFree(junk->tmp_store);
   junk->original_terms->sig = NULL;
   junk->terms->sig = NULL;
   junk->tmp_terms->sig = NULL;
   SigFree(junk->signature);   
   TBFree(junk->original_terms);
   TBFree(junk->terms);
   TBFree(junk->tmp_terms);
   VarBankFree(junk->freshvars);

   ProofStateCellFree(junk);
}


/*-----------------------------------------------------------------------
//
// Function: ProofStateCreateOrdering()
//
//   Given a proof state, generate a (hopefully suitable) ordering for
//   it. 
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

OCB_p ProofStateCreateOrdering(ProofState_p state)
{
   OCB_p handle;

   switch(state->ordertype)
   {      
   case LPO:
	 handle = OCBAlloc(LPO, state->signature);
	 TOGeneratePrecedence(handle, state->axioms, PNoMethod);
	 OUTPRINT(1, "# Creating term ordering: LPO selected\n");
	 break;
   case KBO:
	 handle = OCBAlloc(KBO, state->signature);
	 TOGeneratePrecedence(handle, state->axioms, PNoMethod);
	 TOGenerateWeights(handle, state->axioms,
			   state->to_weight_gen,
			   state->to_const_weight);
	 OUTPRINT(1, "# Creating term ordering: KBO selected\n");
	 break;
   case NoOrdering:
	 /* Do some magic...currently default to KBO2! -> Fall through! */
   case KBO2:
	 handle = OCBAlloc(KBO2, state->signature);
	 TOGeneratePrecedence(handle, state->axioms, PNoMethod);
	 TOGenerateWeights(handle, state->axioms,
			   state->to_weight_gen,
			   state->to_const_weight);
	 OUTPRINT(1, "# Creating term ordering: KBO2 selected\n");
	 break;
   case RPO:
	 assert(false && "RPO not yet implemented!");
	 handle = NULL;
	 break;
   default:
	 assert(false);
	 handle = NULL;
	 break;
   }
   /* Select proper literal ordering for Dershowitz-Hack */
   if(state->paramod_with_units_only)      
   {
      handle->neg_pos_uncomparable = true;
   }
   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: ProofStateStatisticsPrint()
//
//   Print the statistics of the proof state.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ProofStateStatisticsPrint(FILE* out, ProofState_p state)
{
   fprintf(out, "# Initial clauses:                     : %ld\n",
	   state->axioms->members);    
   fprintf(out, "# Processed clauses                    : %ld\n",
	   state->processed_count);
   fprintf(out, "# ...of these trivial                  : %ld\n",
	   state->trivial_count);
   fprintf(out, "# ...subsumed                          : %ld\n",
	   state->forward_subsumed_count);
   fprintf(out, "# ...remaining for further processing  : %ld\n",
	   state->non_trivial_count);
   fprintf(out, "# Backward-subsumed                    : %ld\n",
	   state->backward_subsumed_count);
   fprintf(out, "# Backward-rewritten                   : %ld\n",
	   state->backward_rewritten_count);
   fprintf(out, "# Generated clauses                    : %ld\n",
	   state->generated_count - state->backward_rewritten_count);   
   fprintf(out, "# ...of the previous two non-trivial   : %ld\n",
	   state->non_trivial_generated_count);   
   fprintf(out, "# Paramodulations                      : %ld\n",
	   state->paramod_count);   
   fprintf(out, "# Factorizations                       : %ld\n",
	   state->factor_count);    
   fprintf(out, "# Equation resolutions                 : %ld\n",
	   state->resolv_count);
   fprintf(out, 
	   "# Current number of processed clauses  : %ld\n"
	   "#    Positive orientable unit clauses  : %ld\n"
	   "#    Positive unorientable unit clauses: %ld\n"
	   "#    Negative unit clauses             : %ld\n"
	   "#    Non-unit-clauses                  : %ld\n",
	   state->processed_pos_rules->members+
	   state->processed_pos_eqns->members+
	   state->processed_neg_units->members+
	   state->processed_non_units->members,
	   state->processed_pos_rules->members,
	   state->processed_pos_eqns->members,
	   state->processed_neg_units->members,
	   state->processed_non_units->members);
   fprintf(out, 
	   "# Current number of unprocessed clauses: %ld\n",
	   state->unprocessed->members);
   if(OutputLevel>=2)
   {
      fprintf(out,
	      "# Shared term nodes                    : %ld\n"
	      "# ...corresponding unshared nodes      : %ld\n",
	      TBTermNodes(state->terms),
	      ClauseSetGetTermNodes(state->processed_pos_rules)+
	      ClauseSetGetTermNodes(state->processed_pos_eqns)+
	      ClauseSetGetTermNodes(state->processed_neg_units)+
	      ClauseSetGetTermNodes(state->processed_neg_units)+
	      ClauseSetGetTermNodes(state->unprocessed));      
   }
   /* TermCellStorePrintDistrib(out, state->terms->term_store); */
}

/*-----------------------------------------------------------------------
//
// Function: ProofStatePrint()
//
//   Print the clause sets of the proof state.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ProofStatePrint(FILE* out, ProofState_p state)
{
   fprintf(out, "\n# Processed positive unit clauses:\n");
   ClauseSetPrint(out, state->processed_pos_rules, true);
   ClauseSetPrint(out, state->processed_pos_eqns, true);
   fprintf(out, "\n# Processed negative unit clauses:\n");
   ClauseSetPrint(out, state->processed_neg_units, true);
   fprintf(out, "\n# Processed non-unit clauses:\n");
   ClauseSetPrint(out, state->processed_non_units, true);
   fprintf(out, "\n# Unprocessed clauses:\n");
   ClauseSetPrint(out, state->unprocessed, true);
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/
